// serverChar8Array.cc
/*************************************************************************
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
* National Laboratory, and the Regents of the University of California, as
* Operator of Los Alamos National Laboratory. MPF is distributed
* subject to a Software License Agreement found in file LICENSE that
* is included with this distribution.
*************************************************************************/
#include <stdlib.h>
#include <stddef.h>
#include <string.h>
#include <stdio.h>

#include <epicsThread.h>

#include "Message.h"
#include "Char8ArrayMessage.h"
#include "test.h"

int serverChar8ArrayDebug=0;

class ServerChar8Array {
public:
    ServerChar8Array(const char *name,int queueSize);
    static void server(ServerChar8Array *);
private:
    MessageServer *pMessageServer;
};

ServerChar8Array::ServerChar8Array(const char *name,int queueSize)
{
    pMessageServer = new MessageServer(name,queueSize);
}

static char threadname[] = "serverChar8Array";
extern "C" int initServerChar8Array(const char *serverName,int queueSize)
{
    ServerChar8Array *pServerChar8Array =
        new ServerChar8Array(serverName,queueSize);
    epicsThreadId threadId = epicsThreadCreate(threadname,
        epicsThreadPriorityMedium,
        epicsThreadGetStackSize(epicsThreadStackMedium),
        (EPICSTHREADFUNC)ServerChar8Array::server,(void *)pServerChar8Array);
    if(!threadId) printf("initServerChar8Array epicsThreadCreate Failure\n");
    return(0);
}

void ServerChar8Array::server(ServerChar8Array *pServerChar8Array)
{
    MessageServer *pMessageServer = pServerChar8Array->pMessageServer;
    Message       *message;
    bool allocNew = false;
    while(true) {
	pMessageServer->waitForMessage();
	while((message = pMessageServer->receive())) {
	    if(message->getType()!=messageTypeChar8Array) {
	        printf("serverChar8Array got illegal message type %d\n",
		    message->getType());
	    } else {
		Char8ArrayMessage  *preceive = (Char8ArrayMessage *)message;
		Char8ArrayMessage *psend;
		if(allocNew) {
		    psend = (Char8ArrayMessage *)pMessageServer->
			allocReplyMessage(preceive,messageTypeChar8Array);
		    psend->allocValue(preceive->getSize());
		    ::strcpy(psend->value,preceive->value);
		    psend->setSize(preceive->getSize());
		    delete preceive;
		    allocNew = false;
		} else {
	            psend = preceive;
		    allocNew = true;
		}
		if(serverChar8ArrayDebug) {
		    char buffer[100];

		    sprintf(buffer,"serverChar8Array %s psend %p\n",
			    psend->value,(void *)psend);
		    printf("%s",buffer);
		}
	        pMessageServer->reply(psend);
	    }
	}
    }
}
