//serverInt32.cc
/*************************************************************************
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
* National Laboratory, and the Regents of the University of California, as
* Operator of Los Alamos National Laboratory. MPF is distributed
* subject to a Software License Agreement found in file LICENSE that
* is included with this distribution.
*************************************************************************/

#include <stdlib.h>
#include <stddef.h>
#include <string.h>
#include <stdio.h>

#include <epicsThread.h>

#include "Message.h"
#include "Int32Message.h"
#include "test.h"

class ServerInt32 {
public:
    ServerInt32(const char *name,int queueSize);
    static void server(ServerInt32 *);
private:
    int32    value;
    int32    extra;
    MessageServer *pMessageServer;
};

int serverInt32Debug=0;


ServerInt32::ServerInt32(const char *name,int queueSize)
: value(0)
{
    pMessageServer = new MessageServer(name,queueSize);
}

static char threadname[] = "serverInt32";
extern "C" int initServerInt32(const char *serverName,int queueSize)
{
    ServerInt32 *pServerInt32 = new ServerInt32(serverName,queueSize);
    epicsThreadId threadId = epicsThreadCreate(threadname,
        epicsThreadPriorityMedium,
        epicsThreadGetStackSize(epicsThreadStackMedium),
        (EPICSTHREADFUNC)ServerInt32::server,(void *)pServerInt32);
    if(!threadId) printf("initServerInt32 epicsThreadCreate Failure\n");
    return(0);
}

void ServerInt32::server(ServerInt32 *pServerInt32)
{
    MessageServer *pMessageServer = pServerInt32->pMessageServer;
    pServerInt32->value = 0;
    bool allocNew = false;
    while(true) {
	pMessageServer->waitForMessage();
	Message *message;
	while((message = pMessageServer->receive())) {
	    if(message->getType()!=messageTypeInt32) {
	        printf("serverInt32 got illegal message type %d\n",
		    message->getType());
	    } else {
		Int32Message  *preceive = (Int32Message *)message;
		Int32Message *psend;
		pServerInt32->value = preceive->value;
		pServerInt32->extra = preceive->extra;
		if(allocNew) {
		    psend = (Int32Message *)pMessageServer->
			allocReplyMessage(preceive,messageTypeInt32);
		    delete preceive;
		    allocNew = false;
		    psend->value = pServerInt32->value;
		    psend->extra = pServerInt32->extra;
		} else {
	            psend = preceive;
		    allocNew = true;
		}
		if(serverInt32Debug) {
		    char buffer[100];

		    sprintf(buffer,"server value %d psend %p\n",
			    pServerInt32->value,(void *)psend);
		    printf("%s",buffer);
		}
	        pMessageServer->reply(psend);
	    }
	}
    }
}
