// serialGetConfig.cpp
/*************************************************************************
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
* National Laboratory, and the Regents of the University of California, as
* Operator of Los Alamos National Laboratory. MPF is distributed
* subject to a Software License Agreement found in file LICENSE that
* is included with this distribution.
*************************************************************************/

#include <stdlib.h>
#include <stddef.h>
#include <string.h>
#include <stdio.h>

#include <epicsEvent.h>
#include <epicsThread.h>
#include <cantProceed.h>
#include <iocsh.h>

#include "Message.h"
#include "ConnectMessage.h"
#include "Int32Message.h"
#include "SerialConfigMessage.h"
#include "DLList.h"
#include "serialServer.h"
#include "asynDriver.h"

typedef struct privateData {
    connectStatus connectState;
    epicsEventId sem;
}privateData;

static void myCallback(Message *message,void *clientPvt)
{
    privateData *pprivateData = (privateData *)clientPvt;
    switch(message->getType()) {
    case messageTypeConnect:
	{
	    ConnectMessage *pConnectMessage = (ConnectMessage *)message;

	    printf("getConfigThread callback messageTypeConnect status %s\n",
		connectStatusString[pConnectMessage->status]);
	    pprivateData->connectState = pConnectMessage->status;
	    delete pConnectMessage;
            epicsEventSignal(pprivateData->sem);
	}
	break;
    case messageTypeSerialConfig:
	{
            SerialConfigMessage *pSerialConfigMessage =
                (SerialConfigMessage *)message;
	
            printf("baud %d stopBits %d bitsPerChar %d "
                "parity %c flowControl %c\n",
                (int)pSerialConfigMessage->baud,(int)pSerialConfigMessage->stopBits,
                (int)pSerialConfigMessage->bitsPerChar,
                (char)pSerialConfigMessage->parity,
                (char)pSerialConfigMessage->flowControl);
	    delete pSerialConfigMessage;
            epicsEventSignal(pprivateData->sem);
	}
	break;
    default:
	printf("got illegal response message\n");
	break;
    }
}

typedef struct args {
    char *server;
    int location;
}args;

static void getConfigThread(void *inargs)
{
    args *pargs = (args *)inargs;
    const char *server = pargs->server;
    int location = pargs->location;

    privateData	*pprivateData = new privateData;
    MessageClient *pMessageClient = new MessageClient(myCallback,pprivateData);
    pprivateData->sem  = epicsEventMustCreate(epicsEventEmpty);
printf("server %s  location %d\n",server,location);
    int status = pMessageClient->bind(server,location);
    if(status) {
	printf("bind returned %d\n",status);
	return;
    }
    while(pprivateData->connectState!=connectYes) {
        epicsEventWaitWithTimeout(pprivateData->sem,1.0);
    }
    Int32Message *message = new Int32Message;
    message->cmd = cmdGetConfig;
    status = pMessageClient->send(message);
    if(status) {
        printf("send returned %d\n",status);
        delete message;
        return;
    }
    epicsEventWaitWithTimeout(pprivateData->sem,1.0);
}

int serialGetConfig(const char *server,int location)
{
    args *pargs = (args *)callocMustSucceed(1,sizeof(args),
        "initClientGetConfig");
    pargs->server = (char *)callocMustSucceed(strlen(server)+1,sizeof(char),
        "initClientGetConfig");
    strcpy(pargs->server,server);
    pargs->location = location;
    epicsThreadId threadId;
    threadId = epicsThreadCreate("getConfigThread",epicsThreadPriorityMedium,
        epicsThreadGetStackSize(epicsThreadStackMedium),
        (EPICSTHREADFUNC)getConfigThread,(void *)pargs);
    if(!threadId) printf("initClientGetConfig epicsThreadCreate Failure\n");
    return(0);
}

/* Register initClientGetConfig */

extern "C" {

//serialGetConfig
static const iocshArg serialGetConfigArg0 =
    { "server",iocshArgString};
static const iocshArg serialGetConfigArg1 =
    { "location",iocshArgInt};
static const iocshArg * const serialGetConfigArgs[2] = 
    {&serialGetConfigArg0,&serialGetConfigArg1};
static const iocshFuncDef serialGetConfigFuncDef
    = {"serialGetConfig",2,serialGetConfigArgs};
static void serialGetConfigCallFunc(const iocshArgBuf *args)
{
    serialGetConfig(args[0].sval,args[1].ival);
}

void testRegister(void)
{
    iocshRegister(&serialGetConfigFuncDef,serialGetConfigCallFunc);
}

} // extern "C"

class MpfTestRegister {
public:
    MpfTestRegister() {testRegister();}
};

static MpfTestRegister testRegisterObj;
